/*
 * XBoing - An X11 blockout style computer game
 *
 * (c) Copyright 1993, 1994, 1995, Justin C. Kibell, All Rights Reserved
 *
 * The X Consortium, and any party obtaining a copy of these files from
 * the X Consortium, directly or indirectly, is granted, free of charge, a
 * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
 * nonexclusive right and license to deal in this software and
 * documentation files (the "Software"), including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons who receive
 * copies from any such party to do so.  This license includes without
 * limitation a license to do the foregoing actions under any patents of
 * the party supplying this software to the X Consortium.
 *
 * In no event shall the author be liable to any party for direct, indirect,
 * special, incidental, or consequential damages arising out of the use of
 * this software and its documentation, even if the author has been advised
 * of the possibility of such damage.
 *
 * The author specifically disclaims any warranties, including, but not limited
 * to, the implied warranties of merchantability and fitness for a particular
 * purpose.  The software provided hereunder is on an "AS IS" basis, and the
 * author has no obligation to provide maintenance, support, updates,
 * enhancements, or modifications.
 */

/* 
 * =========================================================================
 *
 * $Id: AMIGAaudio.c,v 1.1.1.1 1994/12/16 01:36:56 jck Exp $
 * $Source: /usr5/legends/jck/xb/master/xboing/audio/AMIGAaudio.c,v $
 * $Revision: 1.1.1.1 $
 * $Date: 1994/12/16 01:36:56 $
 *
 * $Log: AMIGAaudio.c,v $
 * Revision 1.1.1.1  1994/12/16  01:36:56  jck
 * The XBoing distribution requires configuration management. This is why the
 * cvs utility is being used. This is the initial import of all source etc..
 *
 *
 * =========================================================================
 */

/*
 *  Audio support for NetBSD/Amiga
 */

/*
 *  constructed by Gunther Nikl 3/1997
 */

/*
 *  Include file dependencies:
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <sys/audioio.h>

#include "include/error.h"
#include "include/audio.h"

/*
 *  Internal macro definitions:
 */

#define	BSIZE			4096

#define	SUN_AUDIO_MAGIC	((u_long)0x2e736e64)

/*
 *  Internal type declarations:
 */

typedef struct { /* What sun's .au format looks like */
	u_long	magic;
	u_long	hdr_size;
	u_long	data_size;
	u_long	data_encoding;
	u_long	sample_rate;
	u_long	nchannels;
} Sun_audio_hdr;

/*
 *  Internal variable declarations:
 */

static int child_pid = 0;
static int snd_pipes[2];
static int vol_settings[2];

/*
 *  audio functions
 */

#if NeedFunctionPrototypes
int SetUpAudioSystem(Display *display)
#else
int SetUpAudioSystem(display)
	Display *display;
#endif
{
	audio_info_t	audio_info, ai;
	Sun_audio_hdr	ahdr;
	char			errorString[256], buf[BSIZE], *str, *err;
	int				volumes[2], afd, sfd, cnt;

	if (child_pid == 0) {

		/* Fork a child to handle the I/O */
		if (pipe(&snd_pipes[0])) {

			ErrorMessage("Cannot create pipes to communicate with child.");

			/* Bummer - cannot open pipes */
			return False;
		}

		if ((child_pid=fork()) != 0) {

			/* Successfully forked child */
			return True;
		}
		else {

			/* Child Thread */

			if ((str=getenv("XBOING_SOUND_DIR")) == NULL)
				str = SOUNDS_DIR;

			/* play all samples as requested */

			do {

				read(snd_pipes[0],&errorString[0],sizeof(errorString));

				if (!strcmp(&errorString[0],"EXIT")) {
					/* this is the end */
					exit(0);
				}

				read(snd_pipes[0],(char *)&volumes[0],sizeof(volumes));

				err = "Cannot open audio device.";

				if ((afd=open("/dev/audio",O_WRONLY)) != -1) {

					if	(ioctl(afd,AUDIO_GETINFO,&audio_info) != -1) {

						if (!volumes[1])
							volumes[1]=(audio_info.play.gain*100)/AUDIO_MAX_GAIN;

						sprintf(buf,"%s/%s.au",str,errorString);

						err = "Unable to play sound file %s.";

						if (((sfd=open(buf,O_RDONLY,0)) != -1) &&
							(read(sfd,&ahdr,sizeof(ahdr)) == sizeof(ahdr)) &&
							(ahdr.magic == SUN_AUDIO_MAGIC)) {

							/* cannot set up the device */
							err = "Unable to initialize audio device.";

							/* play it safe */
							ai = audio_info;

							/* init device */
							ai.play.sample_rate	= ahdr.sample_rate;
#if 0
							ai.play.channels	= ahdr.nchannels;
#endif
							ai.play.encoding	= ahdr.data_encoding;
							ai.play.gain		= (AUDIO_MAX_GAIN*volumes[0]*volumes[1])/(100*100);
							ai.mode				= AUMODE_PLAY;

							if (ioctl(afd,AUDIO_SETINFO,&ai) != -1) {

								/* advance to sample start */
								lseek(sfd,ahdr.hdr_size,SEEK_SET);

								/* no error */
								err = NULL;

								/* ok, lets play... */
								while ((cnt=read(sfd,buf,BSIZE)) > 0) {
									if (cnt != write(afd,buf,cnt)) {
									  err = "Problem while writing to sound device."; break;
									}
								}
							}
							ioctl(afd, AUDIO_SETINFO, &audio_info);
						}
						close(sfd);
					}
					close(afd);
				}

				if (err != 0) {
					/* inform user */
					sprintf(errorString,err,buf); ErrorMessage(errorString);
				}

			} while (1);
		}
	}
}

#if NeedFunctionPrototypes
void FreeAudioSystem(void)
#else
void FreeAudioSystem()
#endif
{
	if (write(snd_pipes[1],"EXIT",5) < 0)
		ErrorMessage("Cannot write to audio device.");

	child_pid = 0;
}

#if NeedFunctionPrototypes
void setNewVolume(unsigned int Volume)
#else
void setNewVolume(Volume)
	unsigned int Volume;
#endif
{
	/* do nothing here as we don't know what to do! */
}

#if NeedFunctionPrototypes
void playSoundFile(char *filename, int volume)
#else
void playSoundFile(filename, volume)
	char *filename;
	int volume;
#endif
{
	if (!(write(snd_pipes[1],filename,256)<0)) {
		vol_settings[0] = volume;
		if (!(write(snd_pipes[1],(char *)&vol_settings[0],sizeof(vol_settings)))<0)
			return;
	}
	ErrorMessage("Cannot write to audio device.");
}

#if NeedFunctionPrototypes
void audioDeviceEvents(void)
#else
void audioDeviceEvents()
#endif
{
	/* none to do */
}

#if NeedFunctionPrototypes
void SetMaximumVolume(int Volume)
#else
void SetMaximumVolume(Volume)
	int Volume;
#endif
{
	vol_settings[1] = Volume;
}

#if NeedFunctionPrototypes
int GetMaximumVolume(void)
#else
int GetMaximumVolume()
#endif
{
	return vol_settings[1];
}
